@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -46,159 +36,120 @@
 #include "datetime.h"
 #include "cdo_zaxis.h"
 #include "cdi_lockedIO.h"
+#include "string_utilities.h"
 
 void grid_cell_area(int gridID, double *array);
-int getSurfaceID(int vlistID);
+int get_surface_ID(int vlistID);  // from Vertstat.cc
 struct yy_buffer_state *yy_scan_string(const char *str, void *scanner);
 
-static char *
-exprsFromArgument(const std::vector<std::string> &exprArgv)
+static std::string
+exprs_from_argument(const std::vector<std::string> &exprArgv)
 {
-  char *exprs = nullptr;
-  const int exprArgc = exprArgv.size();
-  auto firstArg = exprArgv[0].c_str();
+  std::string exprString{};
 
-  if (exprArgc == 1)
+  if (exprArgv.size() > 0)
     {
-      const size_t slen = strlen(firstArg);
-      exprs = (char *) Malloc(slen + 2);
-      strcpy(exprs, firstArg);
-      if (exprs[slen - 1] != ';')
+      for (size_t i = 0; i < exprArgv.size(); ++i)
         {
-          exprs[slen] = ';';
-          exprs[slen + 1] = 0;
-        }
-    }
-  else if (exprArgc > 1)
-    {
-      size_t slen = exprArgc + 1;
-      for (int i = 0; i < exprArgc; ++i) slen += exprArgv[i].size();
-      exprs = (char *) Malloc(slen);
-      strcpy(exprs, firstArg);
-      char *pexprs = exprs + strlen(firstArg);
-      for (int i = 1; i < exprArgc; ++i)
-        {
-          *pexprs++ = ',';
-          strcpy(pexprs, exprArgv[i].c_str());
-          pexprs += exprArgv[i].size();
-        }
-      if (*pexprs != ';')
-        {
-          *pexprs++ = ';';
-          *pexprs = 0;
+          if (i > 0) exprString += ",";
+          exprString += exprArgv[i];
         }
+      if (exprString[exprString.size()-1] != ';') exprString += ";";
     }
   else
     {
-      operatorCheckArgc(1);
+      operator_check_argc(1);
     }
 
-  return exprs;
+  return exprString;
 }
 
-static char *
-exprsFromFile(const std::vector<std::string> &exprArgv)
+static std::string
+exprs_from_file(const std::vector<std::string> &exprArgv)
 {
-  const int exprArgc = exprArgv.size();
-  if (exprArgc != 1) operatorCheckArgc(1);
+  if (exprArgv.size() != 1) operator_check_argc(1);
   auto exprf = exprArgv[0].c_str();
 
   // Open expr script file for reading
   auto fp = fopen(exprf, "r");
-  if (!fp) cdoAbort("Open failed on %s", exprf);
+  if (!fp) cdo_abort("Open failed on %s", exprf);
 
   struct stat filestat;
-  if (stat(exprf, &filestat) != 0) cdoAbort("Stat failed on %s", exprf);
+  if (stat(exprf, &filestat) != 0) cdo_abort("Stat failed on %s", exprf);
 
-  const size_t fsize = (size_t) filestat.st_size;
-  char *exprs = (char *) Malloc(fsize + 1);
+  const auto fsize = (size_t) filestat.st_size;
 
-  int ichar, ipos = 0;
-  while ((ichar = fgetc(fp)) != EOF) exprs[ipos++] = ichar;
+  std::string exprString{};
+  exprString.reserve(fsize);
+  int ichar;
+  while ((ichar = fgetc(fp)) != EOF) exprString.push_back(ichar);
 
-  exprs[ipos] = 0;
-  if (ipos == 0) cdoAbort("%s is empty!", exprf);
+  if (exprString.size() == 0) cdo_abort("%s is empty!", exprf);
 
   fclose(fp);
 
-  return exprs;
+  return exprString;
 }
 
 #define MAX_PARAMS 4096
 
-static char *
-str_replace(char *target, const char *needle, const char *replacement)
+static std::size_t
+replace_all(std::string &inout, const std::string &what, const std::string &with)
 {
-  char buffer[1024] = { 0 };
-  char *insert_point = &buffer[0];
-  auto tmp = target;
-  const auto needle_len = strlen(needle);
-  const auto repl_len = strlen(replacement);
-
-  while (true)
-    {
-      char *p = strstr(tmp, needle);
-
-      // walked past last occurrence of needle; copy remaining part
-      if (p == nullptr)
-        {
-          strcpy(insert_point, tmp);
-          break;
-        }
-
-      // copy part before needle
-      memcpy(insert_point, tmp, p - tmp);
-      insert_point += p - tmp;
-
-      // copy replacement string
-      memcpy(insert_point, replacement, repl_len);
-      insert_point += repl_len;
-
-      // adjust pointers, move on
-      tmp = p + needle_len;
-    }
-
-  // write altered string back to target
-  strcpy(target, buffer);
-
-  return target;
+    std::size_t count{};
+    for (std::string::size_type pos{};
+         inout.npos != (pos = inout.find(what.data(), pos, what.length()));
+         pos += with.length(), ++count)
+      {
+        inout.replace(pos, what.length(), with.data(), with.length());
+      }
+    return count;
 }
 
-static char *
-exprsExpand(char *exprs, int vlistID)
+static std::string
+exprs_expand(std::string &exprString, const VarList &varList)
 {
-  const auto len = strlen(exprs);
-  unsigned nequal = 0, nsemi = 0;
-  for (size_t i = 0; i < len; ++i)
-    {
-      if (exprs[i] == '=') nequal++;
-      if (exprs[i] == ';') nsemi++;
-    }
+  auto replaceTemplate = false;
+  const std::string templateName = "_ALL_";
 
-  const char *needle = "_T";
-  if (nequal == 0 && nsemi == 1 && strstr(exprs, needle))
+  if (exprString.find(templateName) != std::string::npos)
     {
-      char varname[CDI_MAX_NAME];
-      const auto nvars = vlistNvars(vlistID);
-      const size_t bufsize = 1024;
-      char *sbuf = (char *) Malloc(bufsize);
-      char *buf = (char *) Malloc(nvars * bufsize);
-      buf[0] = 0;
-      for (int varID = 0; varID < nvars; ++varID)
+      replaceTemplate = true;
+      for (size_t varID = 0; varID < varList.size(); ++varID)
         {
-          vlistInqVarName(vlistID, varID, varname);
-          strcpy(sbuf, exprs);
-          str_replace(sbuf, needle, varname);
-          strcat(buf, varname);
-          strcat(buf, "=");
-          strcat(buf, sbuf);
+          if (cdo_cmpstr(templateName.c_str(), varList[varID].name))
+            {
+              replaceTemplate = false;
+              break;
+            }
         }
-      Free(sbuf);
-      Free(exprs);
-      exprs = buf;
     }
 
-  return exprs;
+  if (replaceTemplate)
+    {
+      std::string exprStringNew{};
+      const auto exprStringArgv = cstr_split_with_seperator(exprString.c_str(), ";");
+      for (auto &string : exprStringArgv)
+        {
+          if (string.find(templateName) == std::string::npos)
+            {
+              exprStringNew += string + ";";
+            }
+          else
+            {
+              for (size_t varID = 0; varID < varList.size(); ++varID)
+                {
+                  auto tmpString = string;
+                  replace_all(tmpString, templateName, varList[varID].name);
+                  exprStringNew += tmpString + ";";
+                }
+            }
+        }
+
+      return exprStringNew;
+    }
+
+  return exprString;
 }
 
 static void
@@ -224,7 +175,7 @@ params_init(std::vector<paramType> &params, const VarList &varList, int vlistID)
       params[varID].isValid = true;
       params[varID].select = false;
       params[varID].remove = false;
-      params[varID].hasMissvals = true;
+      params[varID].hasMV = true;
       params[varID].coord = 0;
       params[varID].gridID = varList[varID].gridID;
       params[varID].zaxisID = varList[varID].zaxisID;
@@ -243,7 +194,7 @@ params_init(std::vector<paramType> &params, const VarList &varList, int vlistID)
 }
 
 static void
-params_delete(std::vector<paramType> &params)
+params_delete(const std::vector<paramType> &params)
 {
   for (int varID = 0; varID < MAX_PARAMS; varID++)
     {
@@ -258,7 +209,7 @@ static void
 params_add_coord(parseParamType &parse_arg, int coord, int cdiID, size_t size, const char *units, const char *longname)
 {
   const auto ncoords = parse_arg.ncoords;
-  if (ncoords >= parse_arg.maxcoords) cdoAbort("Too many coordinates (limit=%d)", parse_arg.maxcoords);
+  if (ncoords >= parse_arg.maxcoords) cdo_abort("Too many coordinates (limit=%d)", parse_arg.maxcoords);
 
   parse_arg.coords[ncoords].needed = false;
   parse_arg.coords[ncoords].coord = coord;
@@ -279,7 +230,7 @@ params_add_coord(parseParamType &parse_arg, int coord, int cdiID, size_t size, c
 }
 
 int
-params_get_coordID(parseParamType *parse_arg, int coord, int cdiID)
+params_get_coord_ID(parseParamType *parse_arg, int coord, int cdiID)
 {
   const auto ncoords = parse_arg->ncoords;
   for (int coordID = 0; coordID < ncoords; ++coordID)
@@ -287,7 +238,7 @@ params_get_coordID(parseParamType *parse_arg, int coord, int cdiID)
       if (parse_arg->coords[coordID].coord == coord && parse_arg->coords[coordID].cdiID == cdiID) return coordID;
     }
 
-  cdoAbort("%s: coordinate %c not found!", __func__, coord);
+  cdo_abort("%s: coordinate %c not found!", __func__, coord);
 
   return -1;
 }
@@ -334,7 +285,7 @@ params_add_ts(parseParamType &parse_arg)
   auto &params = parse_arg.params;
 
   const auto varID = parse_arg.nparams;
-  if (varID >= parse_arg.maxparams) cdoAbort("Too many parameter (limit=%d)", parse_arg.maxparams);
+  if (varID >= parse_arg.maxparams) cdo_abort("Too many parameter (limit=%d)", parse_arg.maxparams);
 
   params[varID].name = strdup("_timestep_info");
   params[varID].gridID = parse_arg.pointID;
@@ -394,16 +345,16 @@ genZonalID(int vlistID)
   return zonalID;
 }
 
-static
-void setDateAndTime(paramType &varts, int calendar, int tsID, int64_t vdate0, int vtime0, int64_t vdate, int vtime)
+static void
+setDateAndTime(paramType &varts, int calendar, int tsID, int64_t vdate0, int vtime0, int64_t vdate, int vtime)
 {
   double jdelta = 0.0;
 
   if (tsID)
     {
-      const auto juldate0 = julianDateEncode(calendar, vdate0, vtime0);
-      const auto juldate = julianDateEncode(calendar, vdate, vtime);
-      jdelta = julianDateToSeconds(julianDateSub(juldate, juldate0));
+      const auto juldate0 = juldate_encode(calendar, vdate0, vtime0);
+      const auto juldate = juldate_encode(calendar, vdate, vtime);
+      jdelta = juldate_to_seconds(juldate_sub(juldate, juldate0));
     }
 
   varts.data[CTIMESTEP] = tsID + 1;
@@ -428,17 +379,17 @@ static void
 addOperators(void)
 {
   // clang-format off
-  cdoOperatorAdd("expr",   1, 1, "expressions");
-  cdoOperatorAdd("exprf",  1, 0, "expr script filename");
-  cdoOperatorAdd("aexpr",  0, 1, "expressions");
-  cdoOperatorAdd("aexprf", 0, 0, "expr script filename");
+  cdo_operator_add("expr",   1, 1, "expressions");
+  cdo_operator_add("exprf",  1, 0, "expr script filename");
+  cdo_operator_add("aexpr",  0, 1, "expressions");
+  cdo_operator_add("aexprf", 0, 0, "expr script filename");
   // clang-format on
 }
 
 void *
 Expr(void *process)
 {
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   void *scanner = nullptr;
   yylex_init(&scanner);
@@ -448,30 +399,30 @@ Expr(void *process)
 
   addOperators();
 
-  const auto operatorID = cdoOperatorID();
-  const bool replacesVariables = cdoOperatorF1(operatorID);
-  const bool readsCommandLine = cdoOperatorF2(operatorID);
+  const auto operatorID = cdo_operator_id();
+  const bool replacesVariables = cdo_operator_f1(operatorID);
+  const bool readsCommandLine = cdo_operator_f2(operatorID);
 
-  operatorInputArg(cdoOperatorEnter(operatorID));
+  operator_input_arg(cdo_operator_enter(operatorID));
 
-  auto exprArgv = cdoGetOperArgv();
+  auto exprArgv = cdo_get_oper_argv();
 
-  auto exprs = readsCommandLine ? exprsFromArgument(exprArgv) : exprsFromFile(exprArgv);
+  auto exprString = readsCommandLine ? exprs_from_argument(exprArgv) : exprs_from_file(exprArgv);
 
-  const auto streamID1 = cdoOpenRead(0);
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto streamID1 = cdo_open_read(0);
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
 
   VarList varList1;
   varListInit(varList1, vlistID1);
 
-  exprs = exprsExpand(exprs, vlistID1);
-  if (Options::cdoVerbose) cdoPrint(exprs);
+  exprString = exprs_expand(exprString, varList1);
+  if (Options::cdoVerbose) cdo_print(exprString.c_str());
 
   const auto nvars1 = vlistNvars(vlistID1);
 
   const auto pointID = gridCreate(GRID_GENERIC, 1);
   const auto zonalID = genZonalID(vlistID1);
-  const auto surfaceID = getSurfaceID(vlistID1);
+  const auto surfaceID = get_surface_ID(vlistID1);
 
   parseParamInit(parse_arg, vlistID1, pointID, zonalID, surfaceID);
 
@@ -489,20 +440,20 @@ Expr(void *process)
   params_add_coordinates(vlistID1, parse_arg);
 
   CDO_parser_errorno = 0;
-  yy_scan_string(exprs, scanner);
+  yy_scan_string(exprString.c_str(), scanner);
   yyparse(&parse_arg, scanner);
-  if (CDO_parser_errorno != 0) cdoAbort("Syntax error!");
+  if (CDO_parser_errorno != 0) cdo_abort("Syntax error!");
 
   parse_arg.init = false;
 
   if (Options::cdoVerbose)
     for (int varID = 0; varID < nvars1; varID++)
-      if (parse_arg.needed[varID]) cdoPrint("Needed var: %d %s", varID, params[varID].name);
+      if (parse_arg.needed[varID]) cdo_print("Needed var: %d %s", varID, params[varID].name);
 
   if (Options::cdoVerbose)
     for (int varID = 0; varID < parse_arg.nparams; varID++)
-      cdoPrint("var: %d %s ngp=%zu nlev=%zu coord=%c", varID, params[varID].name, params[varID].ngp, params[varID].nlev,
-               params[varID].coord == 0 ? ' ' : params[varID].coord);
+      cdo_print("var: %d %s ngp=%zu nlev=%zu coord=%c", varID, params[varID].name, params[varID].ngp, params[varID].nlev,
+                params[varID].coord == 0 ? ' ' : params[varID].coord);
 
   std::vector<int> varIDmap(parse_arg.nparams);
 
@@ -524,7 +475,7 @@ Expr(void *process)
             }
         }
     }
-  cdoVlistCopyFlag(vlistID2, vlistID1);  // Copy global attributes
+  cdo_vlist_copy_flag(vlistID2, vlistID1);  // Copy global attributes
 
   //  printf("parse_arg.nparams %d\n", parse_arg.nparams);
   for (int pidx = 0; pidx < parse_arg.nparams; pidx++)
@@ -542,7 +493,7 @@ Expr(void *process)
       const auto varID = vlistDefVar(vlistID2, params[pidx].gridID, params[pidx].zaxisID, params[pidx].steptype);
       cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, params[pidx].name);
       // printf("add: %d %s %d levs %d\n", pidx,  params[pidx].name, varID, zaxisInqSize(params[pidx].zaxisID));
-      if (params[pidx].hasMissvals) vlistDefVarMissval(vlistID2, varID, params[pidx].missval);
+      if (params[pidx].hasMV) vlistDefVarMissval(vlistID2, varID, params[pidx].missval);
       if (params[pidx].units) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, params[pidx].units);
       if (params[pidx].longname) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, params[pidx].longname);
       const auto len = strlen(params[pidx].name);
@@ -561,11 +512,11 @@ Expr(void *process)
     {
       for (int varID = 0; varID < nvars1; varID++)
         if (parse_arg.needed[varID]) printf("needed: %d %s\n", varID, parse_arg.params[varID].name);
-      cdoPrint("vlistNvars(vlistID1)=%d, vlistNvars(vlistID2)=%d", vlistNvars(vlistID1), vlistNvars(vlistID2));
+      cdo_print("vlistNvars(vlistID1)=%d, vlistNvars(vlistID2)=%d", vlistNvars(vlistID1), vlistNvars(vlistID2));
     }
 
   const auto nvars2 = vlistNvars(vlistID2);
-  if (nvars2 == 0) cdoAbort("No output variable found!");
+  if (nvars2 == 0) cdo_abort("No output variable found!");
 
   for (int varID = 0; varID < nvars1; varID++)
     {
@@ -597,7 +548,7 @@ Expr(void *process)
                 {
                   gridID = generate_full_point_grid(gridID);
 
-                  if (!gridHasCoordinates(gridID)) cdoAbort("Cell center coordinates missing!");
+                  if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
 
                   if (coord == 'x') gridInqXvals(gridID, cdata.data());
                   if (coord == 'y') gridInqYvals(gridID, cdata.data());
@@ -614,7 +565,7 @@ Expr(void *process)
                   if (ngp > 1)
                     {
                       const auto wstatus = gridWeights(gridID, cdata.data());
-                      if (wstatus) cdoWarning("Grid cell bounds not available, using constant grid cell area weights!");
+                      if (wstatus) cdo_warning("Grid cell bounds not available, using constant grid cell area weights!");
                     }
                 }
             }
@@ -624,7 +575,7 @@ Expr(void *process)
               const auto nlev = parse_arg.coords[i].size;
               parse_arg.coords[i].data.resize(nlev);
               auto &cdata = parse_arg.coords[i].data;
-              cdoZaxisInqLevels(zaxisID, cdata.data());
+              cdo_zaxis_inq_levels(zaxisID, cdata.data());
             }
           else if (coord == 'd')
             {
@@ -642,7 +593,7 @@ Expr(void *process)
                 }
             }
           else
-            cdoAbort("Computation of coordinate %c not implemented!", coord);
+            cdo_abort("Computation of coordinate %c not implemented!", coord);
         }
     }
 
@@ -653,28 +604,28 @@ Expr(void *process)
         {
           if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w')
             {
-              const auto coordID = params_get_coordID(&parse_arg, coord, params[varID].gridID);
+              const auto coordID = params_get_coord_ID(&parse_arg, coord, params[varID].gridID);
               const auto gridID = parse_arg.coords[coordID].cdiID;
               const auto ngp = parse_arg.coords[coordID].size;
               auto &cdata = parse_arg.coords[coordID].data;
               assert(gridID == params[varID].gridID);
               assert(!cdata.empty());
 
-              arrayCopy(ngp, cdata.data(), params[varID].data);
+              array_copy(ngp, cdata.data(), params[varID].data);
             }
           else if (coord == 'z' || coord == 'd')
             {
-              const auto coordID = params_get_coordID(&parse_arg, coord, params[varID].zaxisID);
+              const auto coordID = params_get_coord_ID(&parse_arg, coord, params[varID].zaxisID);
               const auto zaxisID = parse_arg.coords[coordID].cdiID;
               const auto nlev = parse_arg.coords[coordID].size;
               auto &cdata = parse_arg.coords[coordID].data;
               assert(zaxisID == params[varID].zaxisID);
               assert(!cdata.empty());
 
-              arrayCopy(nlev, cdata.data(), params[varID].data);
+              array_copy(nlev, cdata.data(), params[varID].data);
             }
           else
-            cdoAbort("Computation of coordinate %c not implemented!", coord);
+            cdo_abort("Computation of coordinate %c not implemented!", coord);
         }
     }
 
@@ -684,8 +635,8 @@ Expr(void *process)
   const auto taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  const auto streamID2 = cdoOpenWrite(1);
-  cdoDefVlist(streamID2, vlistID2);
+  const auto streamID2 = cdo_open_write(1);
+  cdo_def_vlist(streamID2, vlistID2);
 
   int64_t vdate0 = 0;
   int vtime0 = 0;
@@ -694,7 +645,7 @@ Expr(void *process)
   int tsID = 0;
   while (true)
     {
-      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
 
       const auto vdate = taxisInqVdate(taxisID1);
@@ -705,11 +656,11 @@ Expr(void *process)
       vdate0 = vdate;
       vtime0 = vtime;
 
-      taxisCopyTimestep(taxisID2, taxisID1);
+      cdo_taxis_copy_timestep(taxisID2, taxisID1);
 
-      cdoDefTimestep(streamID2, tsID);
+      cdo_def_timestep(streamID2, tsID);
 
-      //for (int varID = 0; varID < nvars1; varID++) printf(">>> %s %d\n", params[varID].name, params[varID].isValid);
+      // for (int varID = 0; varID < nvars1; varID++) printf(">>> %s %d\n", params[varID].name, params[varID].isValid);
       for (int varID = 0; varID < nvars1; varID++) params[varID].isValid = true;
       for (int varID = 0; varID < nvars1; varID++)
         if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].nmiss = 0;
@@ -717,13 +668,13 @@ Expr(void *process)
       for (int recID = 0; recID < nrecs; recID++)
         {
           int varID, levelID;
-          cdoInqRecord(streamID1, &varID, &levelID);
+          cdo_inq_record(streamID1, &varID, &levelID);
           if (parse_arg.needed[varID])
             {
               const auto offset = params[varID].ngp * levelID;
               auto vardata = params[varID].data + offset;
               size_t nmiss;
-              cdoReadRecord(streamID1, vardata, &nmiss);
+              cdo_read_record(streamID1, vardata, &nmiss);
               params[varID].nmiss += nmiss;
 
               if (nmiss > 0) cdo_check_missval(params[varID].missval, params[varID].name);
@@ -736,11 +687,11 @@ Expr(void *process)
           if (pidx < nvars1) continue;
 
           params[pidx].nmiss = 0;
-          varrayFill(params[pidx].ngp * params[pidx].nlev, params[pidx].data, 0.0);
+          varray_fill(params[pidx].ngp * params[pidx].nlev, params[pidx].data, 0.0);
         }
 
       parse_arg.cnparams = vartsID + 1;
-      yy_scan_string(exprs, scanner);
+      yy_scan_string(exprString.c_str(), scanner);
       yyparse(&parse_arg, scanner);
 
       for (int varID = 0; varID < nvars2; varID++)
@@ -757,30 +708,28 @@ Expr(void *process)
             {
               const auto offset = ngp * levelID;
               double *vardata = params[pidx].data + offset;
-              const auto nmiss = arrayNumMV(ngp, vardata, missval);
-              cdoDefRecord(streamID2, varID, levelID);
-              cdoWriteRecord(streamID2, vardata, nmiss);
+              const auto nmiss = array_num_mv(ngp, vardata, missval);
+              cdo_def_record(streamID2, varID, levelID);
+              cdo_write_record(streamID2, vardata, nmiss);
             }
         }
 
       tsID++;
     }
 
-  cdoStreamClose(streamID2);
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID2);
+  cdo_stream_close(streamID1);
 
   vlistDestroy(vlistID2);
 
   yylex_destroy(scanner);
 
-  if (exprs) Free(exprs);
-
   params_delete(params);
 
   gridDestroy(pointID);
   if (zonalID != -1) gridDestroy(zonalID);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
