@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2020 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -32,7 +22,6 @@
 #include "pmlist.h"
 #include "param_conversion.h"
 
-
 template <typename T>
 static double
 vert_interp_lev_kernel(float w1, float w2, const T var1L1, const T var1L2, const T missval)
@@ -49,7 +38,7 @@ vert_interp_lev_kernel(float w1, float w2, const T var1L1, const T var1L2, const
 }
 
 constexpr int BottomLevel = 32000;
-constexpr int TopLevel    = 32001;
+constexpr int TopLevel = 32001;
 
 static void
 restore_index_and_weights(int nlev1, int idx, float wgt, int &idx1, int &idx2, float &wgt1, float &wgt2)
@@ -99,7 +88,7 @@ vert_interp_lev(size_t gridsize, int nlev1, T missval, const Varray<T> &vardata1
       auto var2 = &vardata2[gridsize * ilev];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, var2, var1L1, var1L2, missval, wgt1, wgt2)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < gridsize; ++i)
         {
@@ -113,13 +102,9 @@ vert_interp_lev(size_t gridsize, int nlev1, double missval, const Field3D &field
                 const Varray<int> &lev_idx, const Varray<float> &lev_wgt)
 {
   if (field1.memType == MemType::Float)
-    {
-      vert_interp_lev(gridsize, nlev1, (float)missval, field1.vec_f, field2.vec_f, nlev2, lev_idx, lev_wgt);
-    }
+    vert_interp_lev(gridsize, nlev1, (float) missval, field1.vec_f, field2.vec_f, nlev2, lev_idx, lev_wgt);
   else
-    {
-      vert_interp_lev(gridsize, nlev1, missval, field1.vec_d, field2.vec_d, nlev2, lev_idx, lev_wgt);
-    }
+    vert_interp_lev(gridsize, nlev1, missval, field1.vec_d, field2.vec_d, nlev2, lev_idx, lev_wgt);
 }
 
 //  3D vertical interpolation
@@ -134,7 +119,7 @@ vert_interp_lev3d(size_t gridsize, int nlev1, T missval, const Varray<T> &vardat
       auto var2 = &vardata2[offset];
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(gridsize, nlev1, offset, vardata1, var2, lev_idx, lev_wgt, missval)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
       for (size_t i = 0; i < gridsize; i++)
         {
@@ -154,12 +139,10 @@ vert_interp_lev3d(size_t gridsize, int nlev1, T missval, const Varray<T> &vardat
 }
 
 // Explicit instantiation
-template void
-vert_interp_lev3d(size_t gridsize, int nlev1, float missval, const Varray<float> &vardata1, Varray<float> &vardata2, int nlev2,
-                  const Varray<int> &lev_idx, const Varray<float> &lev_wgt);
-template void
-vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Varray<double> &vardata1, Varray<double> &vardata2, int nlev2,
-                  const Varray<int> &lev_idx, const Varray<float> &lev_wgt);
+template void vert_interp_lev3d(size_t gridsize, int nlev1, float missval, const Varray<float> &vardata1, Varray<float> &vardata2,
+                                int nlev2, const Varray<int> &lev_idx, const Varray<float> &lev_wgt);
+template void vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Varray<double> &vardata1,
+                                Varray<double> &vardata2, int nlev2, const Varray<int> &lev_idx, const Varray<float> &lev_wgt);
 
 void
 vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Field3D &field1, Field3D &field2, int nlev2,
@@ -167,7 +150,7 @@ vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Field3D &fie
 {
   if (field1.memType == MemType::Float)
     {
-      vert_interp_lev3d(gridsize, nlev1, (float)missval, field1.vec_f, field2.vec_f, nlev2, lev_idx, lev_wgt);
+      vert_interp_lev3d(gridsize, nlev1, (float) missval, field1.vec_f, field2.vec_f, nlev2, lev_idx, lev_wgt);
     }
   else
     {
@@ -176,7 +159,8 @@ vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Field3D &fie
 }
 
 void
-vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, const std::vector<double> &lev2, Varray<int> &lev_idx, Varray<float> &lev_wgt)
+vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, const Varray<double> &lev2, Varray<int> &lev_idx,
+                 Varray<float> &lev_wgt)
 {
   for (int i2 = 0; i2 < nlev2; ++i2)
     {
@@ -195,7 +179,7 @@ vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, co
           if (lev2[i2] > val1 && lev2[i2] <= val2) break;
         }
 
-      if (i1 == nlev1) cdoAbort("Level %g not found!", lev2[i2]);
+      if (i1 == nlev1) cdo_abort("Level %g not found!", lev2[i2]);
 
       if (i1 - 1 == 0)  // destination levels is not covert by the first two input z levels
         {
@@ -220,7 +204,7 @@ vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, co
 bool
 levelDirUp(const int nlev, const double *const lev)
 {
-  const bool lup = (nlev > 1 && lev[1] > lev[0]);
+  const auto lup = (nlev > 1 && lev[1] > lev[0]);
   for (int k = 1; k < nlev - 1; ++k)
     if (lup && lev[k + 1] <= lev[k]) return false;
 
@@ -230,7 +214,7 @@ levelDirUp(const int nlev, const double *const lev)
 bool
 levelDirDown(const int nlev, const double *const lev)
 {
-  const bool ldown = (nlev > 1 && lev[1] < lev[0]);
+  const auto ldown = (nlev > 1 && lev[1] < lev[0]);
   for (int k = 1; k < nlev - 1; ++k)
     if (ldown && lev[k + 1] >= lev[k]) return false;
 
@@ -251,12 +235,12 @@ vert_gen_weights3d1d(bool expol, size_t gridsize, int nlev1, const Varray<T> &xl
   for (int k = 0; k < nlev1; ++k) lev1p2[0][k] = xlev1[k * gridsize];
   const auto lup = levelDirUp(nlev1, lev1p2[0].data());
   const auto ldown = levelDirDown(nlev1, lev1p2[0].data());
-  if (!lup && !ldown) cdoAbort("Non monotonic zaxis!");
+  if (!lup && !ldown) cdo_abort("Non monotonic zaxis!");
   double level_0 = lup ? -1.e33 : 1.e33;
   double level_N = lup ? 1.e33 : -1.e33;
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(expol, gridsize, nlev1, nlev2, lev2, level_0, level_N, lev1p2, xlev1, xlev_idx, xlev_wgt, lev_idx, lev_wgt)
+#pragma omp parallel for default(shared) schedule(static)
 #endif
   for (size_t i = 0; i < gridsize; i++)
     {
@@ -278,20 +262,16 @@ vert_gen_weights3d1d(bool expol, size_t gridsize, int nlev1, Field3D &field1, in
                      Varray<int> &lev_idx, Varray<float> &lev_wgt)
 {
   if (field1.memType == MemType::Float)
-    {
-      vert_gen_weights3d1d(expol, gridsize, nlev1, field1.vec_f, nlev2, lev2, lev_idx, lev_wgt);
-    }
+    vert_gen_weights3d1d(expol, gridsize, nlev1, field1.vec_f, nlev2, lev2, lev_idx, lev_wgt);
   else
-    {
-      vert_gen_weights3d1d(expol, gridsize, nlev1, field1.vec_d, nlev2, lev2, lev_idx, lev_wgt);
-    }
+    vert_gen_weights3d1d(expol, gridsize, nlev1, field1.vec_d, nlev2, lev2, lev_idx, lev_wgt);
 }
 
 static int
-createZaxisFromZvar(const std::vector<double> &levels, int vlistID, int varID)
+create_zaxis_from_zvar(const std::vector<double> &levels, int vlistID, int varID)
 {
-  int nlevels = levels.size();
-  int zaxisID = zaxisCreate(ZAXIS_GENERIC, nlevels);
+  const auto nlevels = levels.size();
+  const auto zaxisID = zaxisCreate(ZAXIS_GENERIC, nlevels);
 
   char str[CDI_MAX_NAME];
   strcpy(str, "zlev");
@@ -309,10 +289,10 @@ createZaxisFromZvar(const std::vector<double> &levels, int vlistID, int varID)
 }
 
 static int
-createZaxisFromZaxis(const std::vector<double> &levels, int zaxisID1)
+create_zaxis_from_zaxis(const std::vector<double> &levels, int zaxisID1)
 {
-  int nlevels = levels.size();
-  int zaxisID2 = zaxisCreate(zaxisInqType(zaxisID1), nlevels);
+  const auto nlevels = levels.size();
+  const auto zaxisID2 = zaxisCreate(zaxisInqType(zaxisID1), nlevels);
 
   char str[CDI_MAX_NAME];
   int length = CDI_MAX_NAME;
@@ -327,33 +307,37 @@ createZaxisFromZaxis(const std::vector<double> &levels, int zaxisID1)
   cdiInqKeyString(zaxisID1, CDI_GLOBAL, CDI_KEY_UNITS, str, &length);
   if (str[0]) cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_UNITS, str);
 
-  zaxisDefDatatype(zaxisID2, zaxisInqDatatype(zaxisID1));
+  int datatype = CDI_UNDEFID;
+  cdiInqKeyInt(zaxisID1, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
+  cdiDefKeyInt(zaxisID2, CDI_GLOBAL, CDI_KEY_DATATYPE, datatype);
 
   zaxisDefLevels(zaxisID2, levels.data());
 
   return zaxisID2;
 }
 
-static void
-intlevelGetParameter(std::vector<double> &lev2, std::string &zdescription, std::string &zvarname)
+static std::vector<double>
+intlevel_get_parameter(std::string &zdescription, std::string &zvarname)
 {
-  const auto pargc = operatorArgc();
+  std::vector<double> levels;
+
+  const auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto pargv = cdoGetOperArgv();
+      const auto pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = "INTLEVEL";
-      if (kvlist.parseArguments(pargc, pargv) != 0) cdoAbort("Parse error!");
+      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      bool def_level = false;
-      bool def_zdescription = false;
+      auto def_level = false;
+      auto def_zdescription = false;
       for (const auto &kv : kvlist)
         {
           const auto &key = kv.key;
-          //if (kv.nvalues > 1) cdoAbort("Too many values for parameter key >%s<!", key.c_str());
-          if (kv.nvalues < 1) cdoAbort("Missing value for parameter key >%s<!", key.c_str());
+          // if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key.c_str());
+          if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key.c_str());
           const auto &values = kv.values;
           const auto &value = kv.values[0];
           int nvalues = kv.nvalues;
@@ -362,14 +346,14 @@ intlevelGetParameter(std::vector<double> &lev2, std::string &zdescription, std::
           // clang-format off
          if (key == "level")
             {
-              if (def_zdescription) cdoAbort("Parameter level and zdescription can't be mixed!");
-              lev2.resize(nvalues);
-              for (int i = 0; i < nvalues; ++i) lev2[i] = literal2double(values[i]);
+              if (def_zdescription) cdo_abort("Parameter level and zdescription can't be mixed!");
+              levels.resize(nvalues);
+              for (int i = 0; i < nvalues; ++i) levels[i] = literal_to_double(values[i]);
               def_level = true;
             }
           else if (key == "zdescription")
             {
-              if (def_level) cdoAbort("Parameter zdescription and level can't be mixed!");
+              if (def_level) cdo_abort("Parameter zdescription and level can't be mixed!");
               zdescription = value;
               def_zdescription = true;
             }
@@ -377,69 +361,70 @@ intlevelGetParameter(std::vector<double> &lev2, std::string &zdescription, std::
             {
               zvarname = value;
             }
-          else cdoAbort("Invalid parameter key >%s<!", key.c_str());
+          else cdo_abort("Invalid parameter key >%s<!", key.c_str());
           // clang-format on
         }
     }
+
+  return levels;
 }
 
 void *
 Intlevel(void *process)
 {
-  int varID, levelID;
   int zaxisID1 = -1;
   int nlevel = 0;
 
-  cdoInitialize(process);
+  cdo_initialize(process);
 
   // clang-format off
-  const auto INTLEVEL  = cdoOperatorAdd("intlevel",  0, 0, nullptr);
-  const auto INTLEVELX = cdoOperatorAdd("intlevelx", 0, 0, nullptr);
+  const auto INTLEVEL  = cdo_operator_add("intlevel",  0, 0, nullptr);
+  const auto INTLEVELX = cdo_operator_add("intlevelx", 0, 0, nullptr);
   // clang-format on
 
   (void) (INTLEVEL);  // CDO_UNUSED
 
-  const auto operatorID = cdoOperatorID();
+  const auto operatorID = cdo_operator_id();
 
-  const bool expol = (operatorID == INTLEVELX);
+  const auto expol = (operatorID == INTLEVELX);
 
-  operatorInputArg("level|zdescription, [zvarname]");
+  operator_input_arg("level|zdescription, [zvarname]");
 
   std::vector<double> lev2;
   int zaxisID2 = CDI_UNDEFID;
   std::string zvarname;
-  auto argv = cdoGetOperArgv();
+  auto argv = cdo_get_oper_argv();
 
   if (isdigit((int) argv[0][0]))
     {
-      lev2 = cdoArgvToFlt(argv);
+      lev2 = cdo_argv_to_flt(argv);
     }
   else
     {
       std::string zdescription;
-      intlevelGetParameter(lev2, zdescription, zvarname);
+      lev2 = intlevel_get_parameter(zdescription, zvarname);
       if (!zdescription.empty())
         {
           auto zfilename = zdescription.c_str();
           auto zfp = fopen(zfilename, "r");
-          if (!zfp) cdoAbort("Open failed on %s", zfilename);
-          zaxisID2 = zaxisFromFile(zfp, zfilename);
+          if (!zfp) cdo_abort("Open failed on %s", zfilename);
+          zaxisID2 = zaxis_from_file(zfp, zfilename);
           fclose(zfp);
-          if (zaxisID2 == CDI_UNDEFID) cdoAbort("Invalid zaxis description file %s!", zfilename);
+          if (zaxisID2 == CDI_UNDEFID) cdo_abort("Invalid zaxis description file %s!", zfilename);
           const auto nlevels = zaxisInqSize(zaxisID2);
           lev2.resize(nlevels);
-          zaxisInqLevels(zaxisID2, lev2.data());            
+          zaxisInqLevels(zaxisID2, lev2.data());
         }
     }
 
   const int nlev2 = lev2.size();
 
   if (Options::cdoVerbose)
-    for (int i = 0; i < nlev2; ++i) cdoPrint("lev2 %d: %g", i, lev2[i]);
+    for (int i = 0; i < nlev2; ++i) cdo_print("lev2 %d: %g", i, lev2[i]);
 
-  const auto streamID1 = cdoOpenRead(0);
+  const auto streamID1 = cdo_open_read(0);
 
-  const auto vlistID1 = cdoStreamInqVlist(streamID1);
+  const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
   const auto vlistID2 = vlistDuplicate(vlistID1);
 
   const auto taxisID1 = vlistInqTaxis(vlistID1);
@@ -462,7 +447,7 @@ Intlevel(void *process)
   size_t wisize = 0;
   if (!zvarname.empty())
     {
-      for (varID = 0; varID < nvars; varID++)
+      for (int varID = 0; varID < nvars; varID++)
         {
           if (cdo_cmpstr(zvarname.c_str(), varList1[varID].name))
             {
@@ -471,12 +456,12 @@ Intlevel(void *process)
             }
         }
 
-      if (zvarID == CDI_UNDEFID) cdoAbort("Variable %s not found!", zvarname.c_str());
+      if (zvarID == CDI_UNDEFID) cdo_abort("Variable %s not found!", zvarname.c_str());
       zvarIsVarying = varList1[zvarID].timetype == TIME_VARYING;
       zvarGridsize = varList1[zvarID].gridsize;
       nlev1 = varList1[zvarID].nlevels;
 
-      if (zaxisID2 == CDI_UNDEFID) zaxisID2 = createZaxisFromZvar(lev2, vlistID1, zvarID);
+      if (zaxisID2 == CDI_UNDEFID) zaxisID2 = create_zaxis_from_zvar(lev2, vlistID1, zvarID);
 
       wisize = zvarGridsize * nlev2;
 
@@ -492,7 +477,7 @@ Intlevel(void *process)
               break;
             }
         }
-      if (i == nzaxis) cdoAbort("No processable variable found!");
+      if (i == nzaxis) cdo_abort("No processable variable found!");
     }
   else
     {
@@ -502,29 +487,29 @@ Intlevel(void *process)
         {
           const auto zaxisID = vlistZaxis(vlistID1, i);
           nlevel = zaxisInqSize(zaxisID);
-          //if (zaxisInqType(zaxisID) != ZAXIS_HYBRID && zaxisInqType(zaxisID) != ZAXIS_HYBRID_HALF)
-            if (nlevel > 1)
-              {
-                zaxisID1 = zaxisID;
-                break;
-              }
+          // if (zaxisInqType(zaxisID) != ZAXIS_HYBRID && zaxisInqType(zaxisID) != ZAXIS_HYBRID_HALF)
+          if (nlevel > 1)
+            {
+              zaxisID1 = zaxisID;
+              break;
+            }
         }
-      if (i == nzaxis) cdoAbort("No processable variable found!");
+      if (i == nzaxis) cdo_abort("No processable variable found!");
 
-      if (zaxisID2 == CDI_UNDEFID) zaxisID2 = createZaxisFromZaxis(lev2, zaxisID1);
+      if (zaxisID2 == CDI_UNDEFID) zaxisID2 = create_zaxis_from_zaxis(lev2, zaxisID1);
 
       nlev1 = nlevel;
       lev1.resize(nlev1 + 2);
-      cdoZaxisInqLevels(zaxisID1, &lev1[1]);
+      cdo_zaxis_inq_levels(zaxisID1, &lev1[1]);
 
       const auto lup = levelDirUp(nlev1, &lev1[1]);
       const auto ldown = levelDirDown(nlev1, &lev1[1]);
-      if (!lup && !ldown) cdoAbort("Non monotonic zaxis!");
+      if (!lup && !ldown) cdo_abort("Non monotonic zaxis!");
       lev1[0] = lup ? -1.e33 : 1.e33;
       lev1[nlev1 + 1] = lup ? 1.e33 : -1.e33;
 
       if (Options::cdoVerbose)
-        for (i = 0; i < nlev1 + 2; ++i) cdoPrint("lev1 %d: %g", i, lev1[i]);
+        for (i = 0; i < nlev1 + 2; ++i) cdo_print("lev1 %d: %g", i, lev1[i]);
 
       wisize = nlev2;
     }
@@ -534,8 +519,7 @@ Intlevel(void *process)
     {
       const auto zaxisID = vlistZaxis(vlistID1, index);
       const auto nlevels = zaxisInqSize(zaxisID);
-      if (zaxisID == zaxisID1 || nlevels == nlev1)
-        vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
+      if (zaxisID == zaxisID1 || nlevels == nlev1) vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
     }
 
   VarList varList2;
@@ -545,9 +529,9 @@ Intlevel(void *process)
   Varray<int> lev_idx(wisize);
   Varray<float> lev_wgt(wisize);
 
-  const auto streamID2 = cdoOpenWrite(1);
+  const auto streamID2 = cdo_open_write(1);
 
-  cdoDefVlist(streamID2, vlistID2);
+  cdo_def_vlist(streamID2, vlistID2);
 
   std::vector<bool> vars(nvars);
   std::vector<bool> varinterp(nvars);
@@ -556,7 +540,7 @@ Intlevel(void *process)
 
   const int maxlev = (nlev1 > nlev2) ? nlev1 : nlev2;
 
-  for (varID = 0; varID < nvars; varID++)
+  for (int varID = 0; varID < nvars; varID++)
     {
       const auto nlevels = varList1[varID].nlevels;
 
@@ -578,18 +562,19 @@ Intlevel(void *process)
   int tsID = 0;
   while (true)
     {
-      const auto nrecs = cdoStreamInqTimestep(streamID1, tsID);
+      const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
       if (nrecs == 0) break;
 
-      for (varID = 0; varID < nvars; ++varID) vars[varID] = false;
+      for (int varID = 0; varID < nvars; ++varID) vars[varID] = false;
 
-      taxisCopyTimestep(taxisID2, taxisID1);
-      cdoDefTimestep(streamID2, tsID);
+      cdo_taxis_copy_timestep(taxisID2, taxisID1);
+      cdo_def_timestep(streamID2, tsID);
 
       for (int recID = 0; recID < nrecs; recID++)
         {
-          cdoInqRecord(streamID1, &varID, &levelID);
-          cdoReadRecord(streamID1, vardata1[varID], levelID, &varnmiss[varID][levelID]);
+          int varID, levelID;
+          cdo_inq_record(streamID1, &varID, &levelID);
+          cdo_read_record(streamID1, vardata1[varID], levelID, &varnmiss[varID][levelID]);
           vars[varID] = true;
         }
 
@@ -601,7 +586,7 @@ Intlevel(void *process)
             vert_gen_weights(expol, nlev1 + 2, lev1, nlev2, lev2, lev_idx, lev_wgt);
         }
 
-      for (varID = 0; varID < nvars; varID++)
+      for (int varID = 0; varID < nvars; varID++)
         {
           if (vars[varID] && varinterp[varID])
             {
@@ -613,25 +598,26 @@ Intlevel(void *process)
               else
                 vert_interp_lev(gridsize, nlev1, missval, vardata1[varID], vardata2[varID], nlev2, lev_idx, lev_wgt);
 
-              for (levelID = 0; levelID < nlev2; levelID++)
+              for (int levelID = 0; levelID < nlev2; levelID++)
                 {
                   const auto offset = gridsize * levelID;
                   if (memtype == MemType::Float)
-                    varnmiss[varID][levelID] = arrayNumMV(gridsize, &vardata2[varID].vec_f[offset], (float)missval);
+                    varnmiss[varID][levelID] = array_num_mv(gridsize, &vardata2[varID].vec_f[offset], (float) missval);
                   else
-                    varnmiss[varID][levelID] = arrayNumMV(gridsize, &vardata2[varID].vec_d[offset], missval);
+                    varnmiss[varID][levelID] = array_num_mv(gridsize, &vardata2[varID].vec_d[offset], missval);
                 }
             }
         }
 
-      for (varID = 0; varID < nvars; varID++)
+      for (int varID = 0; varID < nvars; varID++)
         {
           if (vars[varID])
             {
-              for (levelID = 0; levelID < varList2[varID].nlevels; levelID++)
+              for (int levelID = 0; levelID < varList2[varID].nlevels; levelID++)
                 {
-                  cdoDefRecord(streamID2, varID, levelID);
-                  cdoWriteRecord(streamID2, varinterp[varID] ? vardata2[varID] : vardata1[varID], levelID, varnmiss[varID][levelID]);
+                  cdo_def_record(streamID2, varID, levelID);
+                  cdo_write_record(streamID2, varinterp[varID] ? vardata2[varID] : vardata1[varID], levelID,
+                                   varnmiss[varID][levelID]);
                 }
             }
         }
@@ -639,10 +625,10 @@ Intlevel(void *process)
       tsID++;
     }
 
-  cdoStreamClose(streamID2);
-  cdoStreamClose(streamID1);
+  cdo_stream_close(streamID2);
+  cdo_stream_close(streamID1);
 
-  cdoFinish();
+  cdo_finish();
 
   return nullptr;
 }
