@@ -1,23 +1,10 @@
 /*
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
+
   This file is a extension of CDO.
-  Created by M. Koutek, 2012, KNMI (NL)
 
-  This file can eventually become a part of CDO.
+  Author: M. Koutek, 2012, KNMI (NL)
 
-  CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
-
-  Copyright (C) 2003-2012 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -416,7 +403,7 @@ DestaggerUV()
 
       if (gridID0 == -1) cdo_abort("Cannot define DESTAGGERED grid for U, V.");
 
-      if (cdoDebugExt >= 10) cdoPrintGrid(gridID0, 1);
+      if (cdoDebugExt >= 10) cdo_print_griddes(gridID0, 1);
 
       double xfirst_R = gridInqXval(gridID0, 0);  // reference grid for non-staggered fields
                                                   // (default: search for temperature; otherwise: create a new grid)
