@@ -1,18 +1,8 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2021 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
-  See COPYING file for copying and redistribution conditions.
+  Author: Uwe Schulzweida
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
 
 /*
@@ -389,7 +379,7 @@ XTimstat(void *process)
           if (nsets == 0)
             {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(maxrecs, recList, curFirst, input_vars, vars1, samp1) if (maxrecs > 1)
+#pragma omp parallel for default(shared) if (maxrecs > 1)
 #endif
               for (int recID = 0; recID < maxrecs; recID++)
                 {
@@ -416,8 +406,7 @@ XTimstat(void *process)
           else
             {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    shared(lvarstd, nsets, maxrecs, recList, curFirst, input_vars, vars1, samp1, vars2, operfunc) if (maxrecs > 1)
+#pragma omp parallel for default(shared) if (maxrecs > 1)
 #endif
               for (int recID = 0; recID < maxrecs; recID++)
                 {
@@ -468,7 +457,7 @@ XTimstat(void *process)
       if (lmean)
         {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(vlistID1, nsets, maxrecs, recList, vars1, samp1) if (maxrecs > 1)
+#pragma omp parallel for default(shared) if (maxrecs > 1)
 #endif
           for (int recID = 0; recID < maxrecs; recID++)
             {
